from django.utils.six.moves import html_parser


class CssCollectorHtmlParser(html_parser.HTMLParser):
    def __init__(self, content):
        html_parser.HTMLParser.__init__(self)
        self.content = content
        self._current_tag = None
        self.css_entries = []
        self.feed(self.content)
        self.close()

    def handle_starttag(self, tag, attrs):
        tag = tag.lower()
        if tag == 'link':
            is_css = False
            url = None
            for name, value in attrs:
                name = name.strip().lower()
                if (name == 'rel' and value.strip().lower() == 'stylesheet'):
                    is_css = True
                if name == 'href':
                    url = value.strip()
            if not is_css:
                return
            self.css_entries.append({'href': url})
        if tag == 'style':
            self.css_entries.append({})
            self._current_tag = tag

    def handle_endtag(self, tag):
        if self._current_tag and self._current_tag == tag.lower():
            self._current_tag = None

    def handle_data(self, data):
        if self._current_tag == 'style':
            self.css_entries[-1]['text'] = data


def extract_css_entries(request, content):
    parser = CssCollectorHtmlParser(content)
    css_entries = parser.css_entries
    for entry in css_entries:
        try:
            entry['href'] = request.build_absolute_uri(entry['href'])
        except KeyError:
            pass
    return css_entries



from __future__ import unicode_literals
import hashlib
import re

from cssmin import cssmin

from django.core.cache import cache
from django.template import Context
from django.template.loader import get_template

from . import (CRITICAL_MARK_BEGIN, CRITICAL_MARK_END, CRITICAL_ASYNC_MARK,
               CRITICAL_KEY_MARK_BEGIN, CRITICAL_KEY_MARK_END)
from .parser import extract_css_entries
from .util import download_css
from .core import get_critical_css


CRITICAL_CSS_RE = re.compile(
    '{begin}(.*?){end}'.format(begin=re.escape(CRITICAL_MARK_BEGIN),
                               end=re.escape(CRITICAL_MARK_END)),
    re.MULTILINE | re.DOTALL)
ASYNC_SNIPPET_RE = re.compile(re.escape(CRITICAL_ASYNC_MARK))
CRITICAL_KEY_RE = re.compile(
    '{begin}(.*?){end}'.format(begin=re.escape(CRITICAL_KEY_MARK_BEGIN),
                               end=re.escape(CRITICAL_KEY_MARK_END)))


class CriticalCssMiddleware(object):
    def process_response(self, request, response):
        if response.streaming:
            return response

        content = response.content

        match = CRITICAL_CSS_RE.search(content)

        if not match:
            return response

        critical_css_fragment = match.group(1)

        key_match = CRITICAL_KEY_RE.search(content)
        if key_match:
            content = CRITICAL_KEY_RE.sub('', content)
            key = key_match.group(1)
        else:
            key = 'default'

        h = hashlib.sha1()
        h.update(critical_css_fragment)
        cache_key = 'django_critical:{hash}:{key}'.format(
            hash=h.hexdigest(), key=key)

        cached = cache.get(cache_key)

        if cached is not None:
            new_fragment, css_entries = cached
        else:
            css_entries = extract_css_entries(request, critical_css_fragment)
            css = download_css(css_entries)
            critical_css = get_critical_css(response.content, css)
            critical_css = cssmin(critical_css)
            new_fragment = '<style>{css}</style>'.format(
                css=critical_css.replace('\\', '\\\\'))
            cache.set(cache_key, (new_fragment, css_entries))

        content = CRITICAL_CSS_RE.sub(new_fragment, content)

        async_snippet_template = get_template('critical/async_snippet.html')
        async_snippet = async_snippet_template.render(Context({
            'critical_css_fragment': critical_css_fragment,
            'css_entries': css_entries,
        }))

        content = ASYNC_SNIPPET_RE.sub(async_snippet, content)

        response.content = content
        return response



from __future__ import unicode_literals
from django import template

from .. import (CRITICAL_MARK_BEGIN, CRITICAL_MARK_END, CRITICAL_ASYNC_MARK,
                CRITICAL_KEY_MARK_BEGIN, CRITICAL_KEY_MARK_END)

register = template.Library()


class CriticalNode(template.Node):

    def __init__(self, nodelist):
        self.nodelist = nodelist

    def get_original_content(self, context):
        return self.nodelist.render(context)

    def render(self, context):
        return ('{begin}{original_content}{end}'
                '<style id="django-critical-async-css-marker"></style>').format(
                    begin=CRITICAL_MARK_BEGIN,
                    end=CRITICAL_MARK_END,
                    original_content=self.get_original_content(context))


@register.tag
def critical(parser, token):
    """
    Extracts and inlines critical path CSS.

    Syntax::

        {% critical %}
        <link rel="stylesheet" href="//maxcdn.bootstrapcdn.com/bootstrap/3.2.0/css/bootstrap.min.css">
        {% endcritical %}

    Which would be rendered something like::

        <style type="text/css">
        h1 { border:5px solid green; }
        </style>
    """

    nodelist = parser.parse(('endcritical',))
    parser.delete_first_token()

    args = token.split_contents()

    if len(args) != 1:
        raise template.TemplateSyntaxError(
            '{tag} tag takes no arguments.'.format(tag=args[0]))

    return CriticalNode(nodelist)


@register.simple_tag
def critical_async():
    return CRITICAL_ASYNC_MARK

@register.simple_tag
def critical_key(key):
    return '{begin}{key}{end}'.format(begin=CRITICAL_KEY_MARK_BEGIN,
                                      end=CRITICAL_KEY_MARK_END,
                                      key=key)



from __future__ import unicode_literals

CRITICAL_MARK_BEGIN = '<!-- BEGIN: django-critical css ('
CRITICAL_MARK_END = ') END: django-critical css -->'
CRITICAL_ASYNC_MARK = '<!-- django-critical async snippet -->'
CRITICAL_KEY_MARK_BEGIN = '<!-- BEGIN: django-critical key ('
CRITICAL_KEY_MARK_END = ') END: django-critical key -->'



import subprocess

from django.conf import settings
from django.core.files.temp import NamedTemporaryFile


class PenthouseCommand(object):
    command = '{phantomjs} {penthouse} {htmlurl} {csspath}'
    encoding = 'utf8'

    def __init__(self, phantomjs=None, penthouse=None):
        self.phantomjs = phantomjs or settings.CRITICAL_PHANTOMJS_PATH
        self.penthouse = penthouse or settings.CRITICAL_PENTHOUSE_PATH

    def run(self, html, css):
        with NamedTemporaryFile(mode='wb', suffix='.html') as htmlfile,\
                NamedTemporaryFile(mode='wb', suffix='.css') as cssfile:
            htmlfile.write(html.encode(self.encoding))
            htmlfile.flush()
            cssfile.write(css)
            cssfile.flush()

            command = self.command.format(
                phantomjs=self.phantomjs,
                penthouse=self.penthouse,
                htmlurl='file:{}'.format(htmlfile.name),
                csspath=cssfile.name)

            proc = subprocess.Popen(command, shell=True,
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE)
            stdout, stderr = proc.communicate()
            proc.wait()
            if proc.returncode:
                raise Exception(
                    'Penthouse command failed ({code}): {message}'.format(
                        code=proc.returncode,
                        message=stderr,
                    ))
            return stdout


def get_critical_css(html, css):
    return PenthouseCommand().run(html, css)



from __future__ import unicode_literals
from django.utils.six.moves import urllib


def download_css(css_entries):
    css = []
    for entry in css_entries:
        try:
            url = entry['href']
            css.append(urllib.request.urlopen(url).read())
        except KeyError:
            pass
        try:
            css.append(entry['text'])
        except KeyError:
            pass
    return ''.join(css)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages


setup(
    name='django-critical',
    version='0.1',
    description=('Inlines critical path CSS and defers loading full CSS '
                 'asynchronously.'),
    author='Martin Blech',
    author_email='martinblech@gmail.com',
    url='https://github.com/martinblech/django-critical',
    long_description=open('README.md', 'r').read(),
    packages=[
        'critical',
        'critical.templatetags',
    ],
    package_data={
        'critical': [
            'templates/critical/*',
        ]
    },
    zip_safe=False,
    install_requires=[
        'django-appconf >= 0.4',
        'cssmin >= 0.2.0',
    ],
    tests_require=[],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
    ],
)
